@@ -82,6 +82,7 @@
 
 from . import signals_common
 from . import pi_sprog_interface
+from . import mqtt_interface
 
 import enum
 import logging
@@ -155,7 +156,7 @@ def map_dcc_signal (sig_id:int,
     global logging
     
     # Do some basic validation on the parameters we have been given
-    logging.info ("Signal "+str(sig_id)+": Creating DCC Address mapping")
+    logging.info ("Signal "+str(sig_id)+": Creating DCC Address mapping for a colour light signal")
     if sig_mapped(sig_id):
         logging.error ("Signal "+str(sig_id)+": Signal already has a DCC Address mapping")
     elif sig_id < 1:
@@ -185,6 +186,7 @@ def map_dcc_signal (sig_id:int,
                 str(signals_common.signal_state_type.DANGER) : danger,                             # Specific to Colour_Light Mappings
                 str(signals_common.signal_state_type.PROCEED) : proceed,                           # Specific to Colour_Light Mappings
                 str(signals_common.signal_state_type.CAUTION) : caution,                           # Specific to Colour_Light Mappings
+                str(signals_common.signal_state_type.CAUTION_APP_CNTL) : caution,                  # Specific to Colour_Light Mappings
                 str(signals_common.signal_state_type.PRELIM_CAUTION) : prelim_caution,             # Specific to Colour_Light Mappings
                 str(signals_common.signal_state_type.FLASH_CAUTION) : flash_caution,               # Specific to Colour_Light Mappings
                 str(signals_common.signal_state_type.FLASH_PRELIM_CAUTION) : flash_prelim_caution, # Specific to Colour_Light Mappings
@@ -237,6 +239,7 @@ def map_traintech_signal (sig_id:int,
                 str(signals_common.signal_state_type.DANGER) : [[base_address,False]],                 # Specific to Colour_Light Mappings
                 str(signals_common.signal_state_type.PROCEED) : [[base_address,True]],                 # Specific to Colour_Light Mappings
                 str(signals_common.signal_state_type.CAUTION) : [[base_address+1,True]],               # Specific to Colour_Light Mappings
+                str(signals_common.signal_state_type.CAUTION_APP_CNTL) : [[base_address+1,True]],      # Specific to Colour_Light Mappings
                 str(signals_common.signal_state_type.PRELIM_CAUTION) : [[base_address+1,False]],       # Specific to Colour_Light Mappings
                 str(signals_common.signal_state_type.FLASH_CAUTION) : [[base_address+3,True]],         # Specific to Colour_Light Mappings
                 str(signals_common.signal_state_type.FLASH_PRELIM_CAUTION) : [[base_address+3,False]], # Specific to Colour_Light Mappings
@@ -357,7 +360,7 @@ def update_dcc_point(point_id:int, state:bool):
     global logging
     
     if point_mapped(point_id):
-        logging.info ("Point "+str(point_id)+": Generating DCC Bus commands to switch point")
+        logging.debug ("Point "+str(point_id)+": Generating DCC Bus commands to switch point")
         # Retrieve the DCC mappings for our point
         dcc_mapping = dcc_point_mappings[str(point_id)]
         if dcc_mapping["reversed"]: state = not state
@@ -370,7 +373,7 @@ def update_dcc_point(point_id:int, state:bool):
 # Function to send the appropriate DCC commands to set the state of a DCC Signal
 #------------------------------------------------------------------------------------------
 
-def update_dcc_signal_aspects(sig_id: int, state: signals_common.signal_state_type):
+def update_dcc_signal_aspects(sig_id: int):
     
     global logging
     
@@ -381,10 +384,17 @@ def update_dcc_signal_aspects(sig_id: int, state: signals_common.signal_state_ty
         if dcc_mapping["mapping_type"] != mapping_type.COLOUR_LIGHT:
             logging.error ("Signal "+str(sig_id)+": Incorrect DCC Mapping Type for signal - Expecting a Colour Light signal")
         else:
-            logging.info ("Signal "+str(sig_id)+": Generating DCC Bus commands to change main signal aspect")
+            logging.debug ("Signal "+str(sig_id)+": Generating DCC Bus commands to change main signal aspect")
             # Send the DCC commands to change the state
-            for entry in dcc_mapping[str(state)]:
-                if entry[0] > 0: pi_sprog_interface.send_accessory_short_event (entry[0],entry[1])
+            for entry in dcc_mapping[str(signals_common.signals[str(sig_id)]["sigstate"])]:
+                if entry[0] > 0:
+                    # Send the DCC commands to change the state via the serial port to the Pi-Sprog.
+                    # Note that the commands will only be sent if the pi-sprog interface is configured
+                    pi_sprog_interface.send_accessory_short_event(entry[0],entry[1])
+                    # Publish the DCC commands to a remote pi-sprog "node" via an external MQTT broker.
+                    # Note that the commands will only be published if networking is configured and
+                    # the node this software is running on is not configured as a "pi-sprog" node
+                    mqtt_interface.publish_accessory_short_event(entry[0],entry[1])        
     return()
 
 #-----------------------------------------------------------------------------------------
@@ -405,9 +415,16 @@ def update_dcc_signal_element (sig_id:int,state:bool, element:str="main_subsidar
         if element != "main_subsidary" and dcc_mapping["mapping_type"] != mapping_type.SEMAPHORE:
             logging.error ("Signal "+str(sig_id)+": Incorrect DCC Mapping Type for signal - Expecting a Semaphore signal")
         else:
-            logging.info ("Signal "+str(sig_id)+": Generating DCC Bus commands to change \'"+element+"\' ")
+            logging.debug ("Signal "+str(sig_id)+": Generating DCC Bus commands to change \'"+element+"\' ")
             # Send the DCC commands to change the state 
-            if dcc_mapping[element] > 0: pi_sprog_interface.send_accessory_short_event (dcc_mapping[element],state)        
+            if dcc_mapping[element] > 0:
+                # Send the DCC commands to change the state via the serial port to the Pi-Sprog.
+                # Note that the commands will only be sent if the pi-sprog interface is configured
+                pi_sprog_interface.send_accessory_short_event(dcc_mapping[element],state)
+                # Publish the DCC commands to a remote pi-sprog "node" via an external MQTT broker.
+                # Note that the commands will only be published if networking is configured and
+                # the node this software is running on is not configured as a "pi-sprog" node
+                mqtt_interface.publish_accessory_short_event(dcc_mapping[element],state)       
     return()
 
 #-----------------------------------------------------------------------------------------
@@ -439,10 +456,17 @@ def update_dcc_signal_route (sig_id:int,route:signals_common.route_type,
             if ( (dcc_mapping["auto_route_inhibit"] and not signal_change) or
                  (not dcc_mapping["auto_route_inhibit"] and signal_change) or
                  (not sig_at_danger and not signal_change) ):
-                logging.info ("Signal "+str(sig_id)+": Generating DCC Bus commands to change route display")
+                logging.debug ("Signal "+str(sig_id)+": Generating DCC Bus commands to change route display")
                 # Send the DCC commands to change the state if required
                 for entry in dcc_mapping[str(route)]:
-                    if entry[0] > 0: pi_sprog_interface.send_accessory_short_event (entry[0],entry[1])
+                    if entry[0] > 0:
+                        # Send the DCC commands to change the state via the serial port to the Pi-Sprog.
+                        # Note that the commands will only be sent if the pi-sprog interface is configured
+                        pi_sprog_interface.send_accessory_short_event(entry[0],entry[1])
+                        # Publish the DCC commands to a remote pi-sprog "node" via an external MQTT broker.
+                        # Note that the commands will only be published if networking is configured and
+                        # the node this software is running on is not configured as a "pi-sprog" node
+                        mqtt_interface.publish_accessory_short_event(entry[0],entry[1])       
     return()
 
 #-----------------------------------------------------------------------------------------
@@ -471,12 +495,19 @@ def update_dcc_signal_theatre (sig_id:int, character_to_display,
         if ( (dcc_mapping["auto_route_inhibit"] and not signal_change) or
              (not dcc_mapping["auto_route_inhibit"] and signal_change) or
              (not sig_at_danger and not signal_change) ):
-            logging.info ("Signal "+str(sig_id)+": Generating DCC Bus commands to change Theatre display")
+            logging.debug ("Signal "+str(sig_id)+": Generating DCC Bus commands to change Theatre display")
             # Send the DCC commands to change the state if required
             for entry in dcc_mapping["THEATRE"]:
                 if entry[0] == character_to_display:
                     for command in entry[1]:
-                        if command[0] > 0: pi_sprog_interface.send_accessory_short_event (command[0],command[1])
+                        if command[0] > 0:
+                            # Send the DCC commands to change the state via the serial port to the Pi-Sprog.
+                            # Note that the commands will only be sent if the pi-sprog interface is configured
+                            pi_sprog_interface.send_accessory_short_event(command[0],command[1])
+                            # Publish the DCC commands to a remote pi-sprog "node" via an external MQTT broker.
+                            # Note that the commands will only be published if networking is configured and
+                            # the node this software is running on is not configured as a "pi-sprog" node
+                            mqtt_interface.publish_accessory_short_event(command[0],command[1])       
     return()
 
 #######################################################################################
